package com.xkrs.controller;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.common.tool.TokenUtil;
import com.xkrs.dao.SysUserDao;
import com.xkrs.model.entity.FirePointEntity;
import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.qo.FirePointQo;
import com.xkrs.service.FirePointService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

/**
 * @author XinYi Song
 */
@RestController
public class FirePointController {

    @Resource
    private FirePointService firePointService;

    @Resource
    private SysUserDao sysUserDao;

    /**
     * 添加火点数据
     * @param firePointQo
     * @return
     */
    @PostMapping("/insertFirePoint")
    public String insertFirePoint(@RequestBody FirePointQo firePointQo){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        FirePointEntity firePointEntity = firePointService.insertFirePoint(firePointQo);
        if(firePointEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"添加数据失败",locale);
        }else {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"添加成功",locale);
        }
    }

    /**
     * 测试
     * @param token
     * @return
     */
    @GetMapping("/selectUserName")
    @PreAuthorize("hasAnyAuthority('auth_general_user')")
    public String selectUserName(@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        // 验证token
        String tokenUserName = TokenUtil.getTokenUserName(token);
        SysUserEntity sysUserEntity = sysUserDao.selectByUserName(tokenUserName);
        if(sysUserEntity == null){
            return outputEncapsulationObject(PromptMessageEnum.USER_LOGIN_ERROR,"您还没有注册登录，请先注册登录",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,sysUserEntity.getUserName(),locale);
    }

    /**
     * 查询今天的好点信息
     * @param token
     * @return
     */
    @GetMapping("/selectTodayFirePoint")
    @PreAuthorize("hasAnyAuthority('auth_general_user')")
    public String selectTodayFirePoint(@RequestHeader(value="Authorization") String token){
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        List<FirePointEntity> firePointEntities = firePointService.selectTodayFirePoint();
        if(firePointEntities == null || firePointEntities.size() == 0){
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE,"暂时还没有火点数据",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,firePointEntities,locale);
    }
}
